-------------------------------------------------------------------------------
-- Title      : firUnit
-- Project    : 
-------------------------------------------------------------------------------
-- File       : operativeUnit.vhd
-- Author     : Jean-Noel BAZIN  <jnbazin@pc-disi-026.enst-bretagne.fr>
-- Company    : 
-- Created    : 2018-04-11
-- Last update: 2018-04-11
-- Platform   : 
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: 8 bit FIR
-------------------------------------------------------------------------------
-- Copyright (c) 2018 
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2018-04-11  1.0      jnbazin Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity firUnit is

  port (
    I_clock               : in  std_logic;  -- global clock
    I_reset               : in  std_logic;  -- asynchronous global reset
    I_inputSample         : in  std_logic_vector(10 downto 0);  -- 8 bit input sample
    I_inputSampleValid    : in  std_logic;
    O_filteredSample      : out std_logic_vector(10 downto 0);  -- filtered sample
    O_filteredSampleValid : out std_logic
    );

end entity firUnit;

architecture archi_firUnit of firUnit is

  component controlUnit is
    port (
    I_clock               : in  std_logic;  -- global clock
    I_reset               : in  std_logic;  -- asynchronous global reset
    I_inputSampleValid    : in  std_logic;  -- Control signal to load the input sample in the sample shift register and shift the register
    I_processingDone      : in  std_logic;
    O_Process             : out  std_logic_vector(1 downto 0);
    O_loadShift           : out std_logic;  -- filtered sample
    O_loadShiftHighPass   : out std_logic;
    O_loadShiftDerivative : out std_logic;
    O_initAddress         : out std_logic;  -- Control signal to initialize register read address
    O_incrAddress         : out std_logic;  -- Control signal to increment register read address
    O_initSum             : out std_logic;  -- Control signal to initialize the MAC register
    O_loadSum             : out std_logic;  -- Control signal to load the MAC register;
    O_loadY               : out std_logic;  -- Control signal to load Y register
    O_FilteredSampleValid : out std_logic  -- Data valid signal for filtered sample
    );
  end component controlUnit;

  component operativeUnit is
    port (
    I_clock          : in  std_logic;   -- global clock
    I_reset          : in  std_logic;   -- asynchronous global reset
    I_inputSample    : in  std_logic_vector(10 downto 0);  -- 8 bit input sample
    I_loadShift      : in  std_logic;  -- Control signal to load the input sample in the sample shift register and shift the register
    I_loadShiftHighPass   : in std_logic;
    I_loadShiftDerivative : in std_logic;
    I_Process             : in  std_logic_vector(1 downto 0);
    I_initAddress    : in  std_logic;  -- Control signal to initialize register read address
    I_incrAddress    : in  std_logic;  -- Control signal to increment register read address
    I_initSum        : in  std_logic;  -- Control signal to initialize the MAC register
    I_loadSum        : in  std_logic;  -- Control signal to load the MAC register;
    I_loadY          : in  std_logic;   -- Control signal to load Y register
    O_processingDone : out std_logic;   -- Indicate that processing is done
    O_Y              : out std_logic_vector(7 downto 0)   -- filtered sample
    );
  end component operativeUnit;

  signal SC_processingDone : std_logic;
  signal SC_loadShift      : std_logic;
  signal SC_loadShiftHighPass :std_logic;
  signal SC_loadShiftDerivative : std_logic;
  signal SC_Process             : std_logic_vector(1 downto 0);
  signal SC_initAddress    : std_logic;
  signal SC_incrAddress    : std_logic;
  signal SC_initSum        : std_logic;
  signal SC_loadSum        : std_logic;
  signal SC_loadY          : std_logic;

begin

  controlUnit_1 : entity work.controlUnit
    port map (
      I_clock               => I_clock,
      I_reset               => I_reset,
      I_inputSampleValid    => I_inputSampleValid,
      I_processingDone      => SC_processingDone,
      O_loadShift           => SC_loadShift,
      O_Process             => SC_Process,
      O_loadShiftHighPass   => SC_loadShiftHighPass,
      O_loadShiftDerivative => SC_loadShiftDerivative,
      O_initAddress         => SC_initAddress,
      O_incrAddress         => SC_incrAddress,
      O_initSum             => SC_initSum,
      O_loadSum             => SC_loadSum,
      O_loadY               => SC_loadY,
      O_FilteredSampleValid => O_FilteredSampleValid);

  operativeUnit_1 : entity work.operativeUnit
    port map (
      I_clock          => I_clock,
      I_reset          => I_reset,
      I_inputSample    => I_inputSample,
      I_loadShift      => SC_loadShift,
      I_loadShiftHighPass  => SC_loadShiftHighPass,
      I_loadShiftDerivative => SC_loadShiftDerivative,
      I_Process        => SC_Process,
      I_initAddress    => SC_initAddress,
      I_incrAddress    => SC_incrAddress,
      I_initSum        => SC_initSum,
      I_loadSum        => SC_loadSum,
      I_loadY          => SC_loadY,
      O_processingDone => SC_processingDone,
      O_Y              => O_filteredSample);
    
      
    

end architecture archi_firUnit;
